@@ -241,6 +241,9 @@ S3_SKIP_SIGNATURE_VALIDATION = is_env_not_false("S3_SKIP_SIGNATURE_VALIDATION")
 # whether to skip waiting for the infrastructure to shut down, or exit immediately
 FORCE_SHUTDOWN = is_env_not_false("FORCE_SHUTDOWN")
 
+# whether the in_docker check should always return true
+OVERRIDE_IN_DOCKER = is_env_true("OVERRIDE_IN_DOCKER")
+
 
 def has_docker():
     try:
@@ -328,6 +331,7 @@ CONFIG_ENV_VARS = [
     "WAIT_FOR_DEBUGGER",
     "KINESIS_INITIALIZE_STREAMS",
     "TF_COMPAT_MODE",
+    "LAMBDA_DOCKER_FLAGS",
 ]
 
 for key, value in six.iteritems(DEFAULT_SERVICE_PORTS):
@@ -355,6 +359,10 @@ def in_docker():
     Returns True if running in a docker container, else False
     Ref. https://docs.docker.com/config/containers/runmetrics/#control-groups
     """
+    if OVERRIDE_IN_DOCKER:
+        return True
+    if os.path.exists("/.dockerenv"):
+        return True
     if not os.path.exists("/proc/1/cgroup"):
         return False
     try:
