@@ -3,7 +3,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = "0.12.15.1"
+VERSION = "0.12.16"
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = "local"
@@ -27,7 +27,12 @@ LOCALHOST_IP = "127.0.0.1"
 LOCALHOST_HOSTNAME = "localhost.localstack.cloud"
 
 # version of the Maven dependency with Java utility code
-LOCALSTACK_MAVEN_VERSION = "0.2.5"
+LOCALSTACK_MAVEN_VERSION = "0.2.14"
+
+THUNDRA_APIKEY_ENV_VAR_NAME = "THUNDRA_APIKEY"
+THUNDRA_APIKEY = os.getenv(THUNDRA_APIKEY_ENV_VAR_NAME)
+THUNDRA_JAVA_AGENT_JAR_NAME = "thundra-agent.jar"
+THUNDRA_JAVA_AGENT_VERSION = os.getenv("THUNDRA_AGENT_VERSION", "LATEST")
 
 # map of default service APIs and ports to be spun up (fetch map from localstack_client)
 DEFAULT_SERVICE_PORTS = localstack_client.config.get_service_ports()
@@ -117,9 +122,7 @@ STEPFUNCTIONS_ZIP_URL = "https://s3.amazonaws.com/stepfunctionslocal/StepFunctio
 KMS_URL_PATTERN = "https://s3-eu-west-2.amazonaws.com/local-kms/localstack/v3/local-kms.<arch>.bin"
 
 # TODO: Temporarily using a fixed version of DDB in Alpine, as we're hitting a SIGSEGV JVM crash with latest
-DYNAMODB_JAR_URL_ALPINE = (
-    "https://github.com/whummer/dynamodb-local/raw/master/etc/DynamoDBLocal.zip"
-)
+DYNAMODB_JAR_URL_ALPINE = "https://github.com/localstack/localstack-artifacts/raw/master/dynamodb-local-patch/etc/DynamoDBLocal.zip"
 DYNAMODB_JAR_URL = "https://s3-us-west-2.amazonaws.com/dynamodb-local/dynamodb_local_latest.zip"
 
 # API endpoint for analytics events
