@@ -58,7 +58,7 @@ from localstack.utils.common import to_bytes, to_str
 LOG = logging.getLogger(__name__)
 
 # regex path patterns
-HOST_REGEX_EXECUTE_API = r"(.*://)?([a-zA-Z0-9-]+)\.execute-api\..*"
+HOST_REGEX_EXECUTE_API = r"(?:.*://)?([a-zA-Z0-9-]+)\.execute-api\..*"
 TARGET_REGEX_S3_URI = (
     r"^arn:aws:apigateway:[a-zA-Z0-9\-]+:s3:path/(?P<bucket>[^/]+)/(?P<object>.+)$"
 )
@@ -259,7 +259,7 @@ def get_api_id_stage_invocation_path(path, headers):
     return api_id, stage, relative_path_w_query_params
 
 
-def invoke_rest_api_from_request(method, path, data, headers, context={}):
+def invoke_rest_api_from_request(method, path, data, headers, context={}, auth_info={}, **kwargs):
     api_id, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(path, headers)
     try:
         return invoke_rest_api(
@@ -271,12 +271,15 @@ def invoke_rest_api_from_request(method, path, data, headers, context={}):
             headers,
             path=path,
             context=context,
+            auth_info=auth_info,
         )
     except AuthorizationError as e:
         return make_error_response("Not authorized to invoke REST API %s: %s" % (api_id, e), 403)
 
 
-def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=None, context={}):
+def invoke_rest_api(
+    api_id, stage, method, invocation_path, data, headers, path=None, context={}, auth_info={}
+):
     path = path or invocation_path
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
 
@@ -321,6 +324,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         context=context,
         resource_id=resource.get("id"),
         response_templates=response_templates,
+        auth_info=auth_info,
     )
 
 
@@ -337,8 +341,8 @@ def invoke_rest_api_integration(
     context={},
     resource_id=None,
     response_templates={},
+    auth_info={},
 ):
-    response = None
     try:
         response = invoke_rest_api_integration_backend(
             api_id,
@@ -353,6 +357,7 @@ def invoke_rest_api_integration(
             context=context,
             resource_id=resource_id,
             response_templates=response_templates,
+            auth_info=auth_info,
         )
         response = apply_response_parameters(response, integration, api_id=api_id)
         return response
@@ -374,6 +379,7 @@ def invoke_rest_api_integration_backend(
     context={},
     resource_id=None,
     response_templates={},
+    auth_info={},
 ):
 
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
@@ -424,6 +430,7 @@ def invoke_rest_api_integration_backend(
                 integration_uri=uri,
                 resource_id=resource_id,
                 resource_path=resource_path,
+                auth_info=auth_info,
             )
             stage_variables = get_stage_variables(api_id, stage)
 
@@ -670,6 +677,7 @@ def get_lambda_event_request_context(
     integration_uri=None,
     resource_id=None,
     resource_path=None,
+    auth_info={},
 ):
     api_id, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(path, headers)
     relative_path, query_string_params = extract_query_string_params(
@@ -678,6 +686,7 @@ def get_lambda_event_request_context(
     source_ip = headers.get("X-Forwarded-For", ",").split(",")[-2].strip()
     integration_uri = integration_uri or ""
     account_id = integration_uri.split(":lambda:path")[-1].split(":function:")[0].split(":")[-1]
+    account_id = account_id or TEST_AWS_ACCOUNT_ID
     domain_name = f"{api_id}.execute-api.{LOCALHOST_HOSTNAME}"
     request_context = {
         # adding stage to the request context path.
@@ -700,6 +709,8 @@ def get_lambda_event_request_context(
         "requestTime": datetime.datetime.utcnow(),
         "requestTimeEpoch": int(time.time() * 1000),
     }
+    if isinstance(auth_info, dict) and auth_info.get("context"):
+        request_context["authorizer"] = auth_info["context"]
     return request_context
 
 
