@@ -13,6 +13,7 @@ import traceback
 import uuid
 from datetime import datetime
 from io import BytesIO
+from threading import BoundedSemaphore
 
 from flask import Flask, Response, jsonify, request
 from six.moves import cStringIO as StringIO
@@ -233,8 +234,9 @@ def build_mapping_obj(data):
     mapping["FunctionArn"] = func_arn(function_name)
     mapping["LastProcessingResult"] = "OK"
     mapping["StateTransitionReason"] = "User action"
-    mapping["LastModified"] = format_timestamp()
+    mapping["LastModified"] = format_timestamp_for_event_source_mapping()
     mapping["State"] = "Enabled" if enabled in [True, None] else "Disabled"
+    mapping["ParallelizationFactor"] = data.get("ParallelizationFactor") or 1
     if "SelfManagedEventSource" in data:
         source_arn = data["SourceAccessConfigurations"][0]["URI"]
         mapping["SelfManagedEventSource"] = data["SelfManagedEventSource"]
@@ -254,6 +256,11 @@ def format_timestamp(timestamp=None):
     return isoformat_milliseconds(timestamp) + "+0000"
 
 
+def format_timestamp_for_event_source_mapping():
+    # event source mappings seem to use a different time format (required for Terraform compat.)
+    return datetime.utcnow().timestamp()
+
+
 def add_event_source(data):
     region = LambdaRegion.get()
     mapping = build_mapping_obj(data)
@@ -264,7 +271,6 @@ def add_event_source(data):
 def update_event_source(uuid_value, data):
     region = LambdaRegion.get()
     function_name = data.get("FunctionName") or ""
-    batch_size = None
     enabled = data.get("Enabled", True)
     for mapping in region.event_source_mappings:
         if uuid_value == mapping["UUID"]:
@@ -281,7 +287,7 @@ def update_event_source(uuid_value, data):
                     mapping["EventSourceArn"], batch_size or mapping["BatchSize"]
                 )
             mapping["State"] = "Enabled" if enabled in [True, None] else "Disabled"
-            mapping["LastModified"] = format_timestamp()
+            mapping["LastModified"] = format_timestamp_for_event_source_mapping()
             mapping["BatchSize"] = batch_size
             if "SourceAccessConfigurations" in (mapping and data):
                 mapping["SourceAccessConfigurations"] = data["SourceAccessConfigurations"]
@@ -445,10 +451,11 @@ def process_kinesis_records(records, stream_name):
         for source in sources:
             arn = source["FunctionArn"]
             for chunk in chunks(records, source["BatchSize"]):
+                shard_id = "shardId-000000000000"
                 event = {
                     "Records": [
                         {
-                            "eventID": "shardId-000000000000:{0}".format(rec["sequenceNumber"]),
+                            "eventID": "{0}:{1}".format(shard_id, rec["sequenceNumber"]),
                             "eventSourceARN": stream_arn,
                             "eventSource": "aws:kinesis",
                             "eventVersion": "1.0",
@@ -462,11 +469,18 @@ def process_kinesis_records(records, stream_name):
                         for rec in chunk
                     ]
                 }
+                lock_discriminator = None
+                if not config.SYNCHRONOUS_KINESIS_EVENTS:
+                    lock_discriminator = f"{stream_arn}/{shard_id}"
+                    lambda_executors.LAMBDA_ASYNC_LOCKS.assure_lock_present(
+                        lock_discriminator, BoundedSemaphore(source["ParallelizationFactor"])
+                    )
                 run_lambda(
                     func_arn=arn,
                     event=event,
                     context={},
                     asynchronous=not config.SYNCHRONOUS_KINESIS_EVENTS,
+                    lock_discriminator=lock_discriminator,
                 )
     except Exception as e:
         LOG.warning(
@@ -702,6 +716,7 @@ def run_lambda(
     suppress_output=False,
     asynchronous=False,
     callback=None,
+    lock_discriminator: str = None,
 ):
     region_name = func_arn.split(":")[3]
     region = LambdaRegion.get(region_name)
@@ -734,6 +749,7 @@ def run_lambda(
             version=version,
             asynchronous=asynchronous,
             callback=callback,
+            lock_discriminator=lock_discriminator,
         )
 
     except Exception as e:
@@ -2236,3 +2252,29 @@ def serve(port):
     LAMBDA_EXECUTOR.startup()
 
     generic_proxy.serve_flask_app(app=app, port=port)
+
+
+# Config listener
+def on_config_change(config_key: str, config_newvalue: str) -> None:
+    global LAMBDA_EXECUTOR
+    if config_key != "LAMBDA_EXECUTOR":
+        return
+    LOG.debug(
+        "Recieved config event for lambda executor! Key: {} Value: {}".format(
+            config_key, config_newvalue
+        )
+    )
+    LAMBDA_EXECUTOR.cleanup()
+    LAMBDA_EXECUTOR = lambda_executors.AVAILABLE_EXECUTORS.get(
+        config_newvalue, lambda_executors.DEFAULT_EXECUTOR
+    )
+    LAMBDA_EXECUTOR.startup()
+
+
+def register_config_listener():
+    from localstack.utils import config_listener
+
+    config_listener.CONFIG_LISTENERS.append(on_config_change)
+
+
+register_config_listener()
