@@ -1,4 +1,5 @@
 import base64
+import contextlib
 import glob
 import json
 import logging
@@ -10,9 +11,14 @@ import threading
 import time
 import traceback
 from multiprocessing import Process, Queue
-from typing import Tuple
+from typing import Dict, Tuple, Union
 
 from localstack import config
+from localstack.constants import (
+    THUNDRA_APIKEY,
+    THUNDRA_APIKEY_ENV_VAR_NAME,
+    THUNDRA_JAVA_AGENT_JAR_NAME,
+)
 from localstack.services.awslambda.lambda_utils import (
     LAMBDA_RUNTIME_JAVA8,
     LAMBDA_RUNTIME_JAVA11,
@@ -31,7 +37,6 @@ from localstack.utils.common import (
     TMP_FILES,
     CaptureOutput,
     FuncThread,
-    cp_r,
     get_docker_container_names,
     get_free_tcp_port,
     in_docker,
@@ -39,7 +44,6 @@ from localstack.utils.common import (
     last_index_of,
     long_uid,
     now,
-    rm_rf,
     run,
     save_file,
     short_uid,
@@ -51,6 +55,9 @@ from localstack.utils.docker import DOCKER_CLIENT, ContainerException
 # constants
 LAMBDA_EXECUTOR_JAR = INSTALL_PATH_LOCALSTACK_FAT_JAR
 LAMBDA_EXECUTOR_CLASS = "cloud.localstack.LambdaExecutor"
+LAMBDA_HANDLER_ENV_VAR_NAME = "_HANDLER"
+THUNDRA_JAVA_AGENT_JAR = "%s/%s" % (config.HOST_TMP_FOLDER, THUNDRA_JAVA_AGENT_JAR_NAME)
+THUNDRA_JAVA_AGENT_CONTAINER_PATH = "/tmp/thundra-agent.jar"
 EVENT_FILE_PATTERN = "%s/lambda.event.*.json" % config.TMP_FOLDER
 
 LAMBDA_SERVER_UNIQUE_PORTS = 500
@@ -101,6 +108,11 @@ def is_java_lambda(lambda_details):
     return runtime in [LAMBDA_RUNTIME_JAVA8, LAMBDA_RUNTIME_JAVA11]
 
 
+def is_java8_lambda(lambda_details):
+    runtime = getattr(lambda_details, "runtime", lambda_details)
+    return runtime == LAMBDA_RUNTIME_JAVA8
+
+
 def is_nodejs_runtime(lambda_details):
     runtime = getattr(lambda_details, "runtime", lambda_details) or ""
     return runtime.startswith("nodejs")
@@ -149,6 +161,24 @@ def rm_docker_container(container_name_or_id, check_existence=False, safe=False)
             raise
 
 
+class LambdaAsyncLocks:
+    locks: Dict[str, Union[threading.Semaphore, threading.Lock]]
+    creation_lock: threading.Lock
+
+    def __init__(self):
+        self.locks = {}
+        self.creation_lock = threading.Lock()
+
+    def assure_lock_present(
+        self, key: str, lock: Union[threading.Semaphore, threading.Lock]
+    ) -> Union[threading.Semaphore, threading.Lock]:
+        with self.creation_lock:
+            return self.locks.setdefault(key, lock)
+
+
+LAMBDA_ASYNC_LOCKS = LambdaAsyncLocks()
+
+
 class InvocationResult(object):
     def __init__(self, result, log_output=""):
         if isinstance(result, InvocationResult):
@@ -175,6 +205,16 @@ class LambdaExecutor(object):
 
         return result
 
+    def _get_thundra_apikey(self, env_vars):
+        thundra_apikey = env_vars.get(THUNDRA_APIKEY_ENV_VAR_NAME)
+
+        # If Thundra API key is specified for the function through env vars, use it
+        if not thundra_apikey:
+            # Otherwise, try to get it from Localstack env vars
+            thundra_apikey = THUNDRA_APIKEY
+
+        return thundra_apikey
+
     def execute(
         self,
         func_arn,
@@ -184,41 +224,47 @@ class LambdaExecutor(object):
         version=None,
         asynchronous=False,
         callback=None,
+        lock_discriminator: str = None,
     ):
         def do_execute(*args):
             @cloudwatched("lambda")
             def _run(func_arn=None):
-                # set the invocation time in milliseconds
-                invocation_time = int(time.time() * 1000)
-                # start the execution
-                raised_error = None
-                result = None
-                dlq_sent = None
-                try:
-                    result = self._execute(func_arn, func_details, event, context, version)
-                except Exception as e:
-                    raised_error = e
-                    if asynchronous:
-                        if get_from_event(event, "eventSource") == EVENT_SOURCE_SQS:
-                            sqs_queue_arn = get_from_event(event, "eventSourceARN")
-                            if sqs_queue_arn:
-                                # event source is SQS, send event back to dead letter queue
-                                dlq_sent = sqs_error_to_dead_letter_queue(sqs_queue_arn, event, e)
-                        else:
-                            # event source is not SQS, send back to lambda dead letter queue
-                            lambda_error_to_dead_letter_queue(func_details, event, e)
-                    raise e
-                finally:
-                    self.function_invoke_times[func_arn] = invocation_time
-                    callback and callback(
-                        result, func_arn, event, error=raised_error, dlq_sent=dlq_sent
-                    )
-                    lambda_result_to_destination(
-                        func_details, event, result, asynchronous, raised_error
-                    )
+                with contextlib.ExitStack() as stack:
+                    if lock_discriminator:
+                        stack.enter_context(LAMBDA_ASYNC_LOCKS.locks[lock_discriminator])
+                    # set the invocation time in milliseconds
+                    invocation_time = int(time.time() * 1000)
+                    # start the execution
+                    raised_error = None
+                    result = None
+                    dlq_sent = None
+                    try:
+                        result = self._execute(func_arn, func_details, event, context, version)
+                    except Exception as e:
+                        raised_error = e
+                        if asynchronous:
+                            if get_from_event(event, "eventSource") == EVENT_SOURCE_SQS:
+                                sqs_queue_arn = get_from_event(event, "eventSourceARN")
+                                if sqs_queue_arn:
+                                    # event source is SQS, send event back to dead letter queue
+                                    dlq_sent = sqs_error_to_dead_letter_queue(
+                                        sqs_queue_arn, event, e
+                                    )
+                            else:
+                                # event source is not SQS, send back to lambda dead letter queue
+                                lambda_error_to_dead_letter_queue(func_details, event, e)
+                        raise e
+                    finally:
+                        self.function_invoke_times[func_arn] = invocation_time
+                        callback and callback(
+                            result, func_arn, event, error=raised_error, dlq_sent=dlq_sent
+                        )
+                        lambda_result_to_destination(
+                            func_details, event, result, asynchronous, raised_error
+                        )
 
-                # return final result
-                return result
+                    # return final result
+                    return result
 
             return _run(func_arn=func_arn)
 
@@ -255,11 +301,13 @@ class LambdaExecutorContainers(LambdaExecutor):
     """Abstract executor class for executing Lambda functions in Docker containers"""
 
     def execute_in_container(
-        self, func_details, env_vars, command, stdin=None, background=False
+        self, func_details, env_vars, command, docker_flags=None, stdin=None, background=False
     ) -> Tuple[bytes, bytes]:
         raise NotImplementedError
 
-    def run_lambda_executor(self, event=None, func_details=None, env_vars=None, command=None):
+    def run_lambda_executor(
+        self, event=None, func_details=None, env_vars=None, command=None, docker_flags=None
+    ):
         env_vars = dict(env_vars or {})
         runtime = func_details.runtime or ""
 
@@ -307,7 +355,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         error = None
         try:
             result, log_output = self.execute_in_container(
-                func_details, env_vars, command, event_stdin_bytes
+                func_details, env_vars, command, docker_flags=docker_flags, stdin=event_stdin_bytes
             )
         except ContainerException as e:
             result = e.stdout or ""
@@ -357,11 +405,48 @@ class LambdaExecutorContainers(LambdaExecutor):
         environment["AWS_LAMBDA_EVENT_BODY"] = event_body
         return event_body.encode()
 
+    def inject_thundra_java_agent(self, environment, runtime):
+        if not THUNDRA_JAVA_AGENT_JAR:
+            return
+
+        thundra_apikey = super(LambdaExecutorContainers, self)._get_thundra_apikey(environment)
+        if not thundra_apikey:
+            return
+
+        if config.LAMBDA_REMOTE_DOCKER:
+            LOG.info(
+                "Not enabling Thundra agent, as Docker file mounting is disabled due to LAMBDA_REMOTE_DOCKER=1"
+            )
+            return
+
+        environment[THUNDRA_APIKEY_ENV_VAR_NAME] = thundra_apikey
+
+        # Inject Thundra agent path into "JAVA_TOOL_OPTIONS" env var,
+        # so it will be automatically loaded on JVM startup
+        java_tool_opts = environment.get("JAVA_TOOL_OPTIONS", "")
+        java_tool_opts += " -javaagent:" + THUNDRA_JAVA_AGENT_CONTAINER_PATH
+        environment["JAVA_TOOL_OPTIONS"] = java_tool_opts.strip()
+
+        # Disable CDS (Class Data Sharing),
+        # because "-javaagent" cannot be enabled when CDS is enabled on JDK 8.
+        # CDS can only be disabled by "_JAVA_OPTIONS" env var,
+        # because by default it is enabled ("-Xshare:on")
+        # on Lambci by command line parameters and
+        # "_JAVA_OPTIONS" has precedence over command line parameters
+        # but "JAVA_TOOL_OPTIONS" is not.
+        if is_java8_lambda(runtime):
+            java_opts = environment.get("_JAVA_OPTIONS", "")
+            java_opts += " -Xshare:off"
+            environment["_JAVA_OPTIONS"] = java_opts.strip()
+
+        # Mount Thundra agent jar into container file system
+        return "-v %s:%s" % (THUNDRA_JAVA_AGENT_JAR, THUNDRA_JAVA_AGENT_CONTAINER_PATH)
+
     def _execute(self, func_arn, func_details, event, context=None, version=None):
-        lambda_cwd = func_details.cwd
         runtime = func_details.runtime
         handler = func_details.handler
         environment = self._prepare_environment(func_details)
+        given_docker_flags = config.LAMBDA_DOCKER_FLAGS
 
         # configure USE_SSL in environment
         if config.USE_SSL:
@@ -378,7 +463,7 @@ class LambdaExecutorContainers(LambdaExecutor):
 
         environment["LOCALSTACK_HOSTNAME"] = main_endpoint
         environment["EDGE_PORT"] = str(config.EDGE_PORT)
-        environment["_HANDLER"] = handler
+        environment[LAMBDA_HANDLER_ENV_VAR_NAME] = handler
         if os.environ.get("HTTP_PROXY"):
             environment["HTTP_PROXY"] = os.environ["HTTP_PROXY"]
         if func_details.timeout:
@@ -393,57 +478,35 @@ class LambdaExecutorContainers(LambdaExecutor):
                     to_str(base64.b64decode(to_bytes(context.client_context)))
                 )
 
-        # custom command to execute in the container
-        command = ""
-        events_file_path = ""
-
+        # pass JVM options to the Lambda environment, if configured
         if config.LAMBDA_JAVA_OPTS and is_java_lambda(runtime):
-            # if running a Java Lambda with our custom executor, set up classpath arguments
-            java_opts = Util.get_java_opts()
-            stdin = None
-            # copy executor jar into temp directory
-            target_file = os.path.join(lambda_cwd, os.path.basename(LAMBDA_EXECUTOR_JAR))
-            if not os.path.exists(target_file):
-                cp_r(LAMBDA_EXECUTOR_JAR, target_file)
-            # TODO cleanup once we have custom Java Docker image
-            events_file = "_lambda.events.%s.json" % short_uid()
-            events_file_path = os.path.join(lambda_cwd, events_file)
-            save_file(events_file_path, event_body)
-            # construct Java command
-            classpath = Util.get_java_classpath(target_file)
-            command = [
-                "bash",
-                "-c",
-                'cd %s; java %s -cp "%s" "%s" "%s" "%s"'
-                % (
-                    DOCKER_TASK_FOLDER,
-                    java_opts,
-                    classpath,
-                    LAMBDA_EXECUTOR_CLASS,
-                    handler,
-                    events_file,
-                ),
-            ]
+            if environment.get("JAVA_TOOL_OPTIONS"):
+                LOG.info(
+                    "Skip setting LAMBDA_JAVA_OPTS as JAVA_TOOL_OPTIONS already defined in Lambda env vars"
+                )
+            else:
+                LOG.debug(
+                    "Passing JVM options to container environment: JAVA_TOOL_OPTIONS=%s"
+                    % config.LAMBDA_JAVA_OPTS
+                )
+                environment["JAVA_TOOL_OPTIONS"] = config.LAMBDA_JAVA_OPTS
+
+        docker_flags = given_docker_flags or ""
+        if is_java_lambda(runtime):
+            # If runtime is Java, inject Thundra agent if it is configured
+            extra_docker_flags = self.inject_thundra_java_agent(environment, runtime)
+            docker_flags += " %s" % extra_docker_flags if extra_docker_flags else ""
 
         # accept any self-signed certificates for outgoing calls from the Lambda
         if is_nodejs_runtime(runtime):
             environment["NODE_TLS_REJECT_UNAUTHORIZED"] = "0"
 
-        # copy events file into container, if necessary
-        if events_file_path:
-            container_name = self.get_container_name(func_details.arn())
-            self.copy_into_container(events_file_path, container_name, DOCKER_TASK_FOLDER)
-
         # run Lambda executor and fetch invocation result
         LOG.info("Running lambda: %s" % func_details.arn())
         result = self.run_lambda_executor(
-            event=stdin, env_vars=environment, func_details=func_details, command=command
+            event=stdin, env_vars=environment, func_details=func_details, docker_flags=docker_flags
         )
 
-        # clean up events file
-        events_file_path and os.path.exists(events_file_path) and rm_rf(events_file_path)
-        # TODO: delete events file from container!
-
         return result
 
 
@@ -464,7 +527,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         self.port_offset = LAMBDA_SERVER_PORT_OFFSET
 
     def execute_in_container(
-        self, func_details, env_vars, command, stdin=None, background=False
+        self, func_details, env_vars, command, docker_flags=None, stdin=None, background=False
     ) -> Tuple[bytes, bytes]:
         func_arn = func_details.arn()
         lambda_cwd = func_details.cwd
@@ -485,7 +548,9 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             runtime,
             func_arn,
         )
-        container_info = self.prime_docker_container(func_details, dict(env_vars), lambda_cwd)
+        container_info = self.prime_docker_container(
+            func_details, dict(env_vars), lambda_cwd, docker_flags
+        )
 
         if not command and handler:
             command = container_info.entry_point.split()
@@ -526,7 +591,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         self.function_invoke_times = {}
         return self.destroy_existing_docker_containers()
 
-    def prime_docker_container(self, func_details, env_vars, lambda_cwd):
+    def prime_docker_container(self, func_details, env_vars, lambda_cwd, docker_flags=None):
         """
         Prepares a persistent docker container for a specific function.
         :param func_details: The Details of the lambda function.
@@ -551,7 +616,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
                 # get container startup command and run it
                 LOG.debug("Creating container: %s" % container_name)
-                self.create_container(func_details, env_vars, lambda_cwd)
+                self.create_container(func_details, env_vars, lambda_cwd, docker_flags)
 
                 if config.LAMBDA_REMOTE_DOCKER:
                     LOG.debug(
@@ -576,7 +641,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
             return ContainerInfo(container_name, entry_point)
 
-    def create_container(self, func_details, env_vars, lambda_cwd):
+    def create_container(self, func_details, env_vars, lambda_cwd, docker_flags=None):
         docker_image = Util.docker_image_for_lambda(func_details)
         container_name = self.get_container_name(func_details.arn())
 
@@ -589,7 +654,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         env_vars.pop("AWS_LAMBDA_EVENT_BODY", None)
 
         network = config.LAMBDA_DOCKER_NETWORK
-        additional_flags = config.LAMBDA_DOCKER_FLAGS
+        additional_flags = docker_flags
         dns = config.LAMBDA_DOCKER_DNS
 
         mount_volumes = not config.LAMBDA_REMOTE_DOCKER
@@ -742,7 +807,7 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
         return event_body.encode()
 
     def execute_in_container(
-        self, func_details, env_vars, command, stdin=None, background=False
+        self, func_details, env_vars, command, docker_flags=None, stdin=None, background=False
     ) -> Tuple[bytes, bytes]:
         lambda_cwd = func_details.cwd
         handler = func_details.handler
@@ -762,7 +827,7 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
             env_vars["DOCKER_LAMBDA_API_PORT"] = port
             env_vars["DOCKER_LAMBDA_RUNTIME_PORT"] = port
 
-        additional_flags = config.LAMBDA_DOCKER_FLAGS or ""
+        additional_flags = docker_flags or ""
         dns = config.LAMBDA_DOCKER_DNS
         docker_java_ports = bootstrap.PortMappings()
         if Util.debug_java_port:
@@ -818,7 +883,9 @@ class LambdaExecutorLocal(LambdaExecutor):
         :return: the InvocationResult
         """
 
-        kwargs = {"stdin": True, "inherit_env": True, "asynchronous": True}
+        env_vars = func_details and func_details.envvars
+        kwargs = {"stdin": True, "inherit_env": True, "asynchronous": True, "env_vars": env_vars}
+
         process = run(cmd, stderr=subprocess.PIPE, outfile=subprocess.PIPE, **kwargs)
         result, log_output = process.communicate()
         try:
@@ -932,33 +999,55 @@ class LambdaExecutorLocal(LambdaExecutor):
         invocation_result = InvocationResult(result, log_output=log_output)
         return invocation_result
 
+    def inject_thundra_java_agent(self, func_details, given_opts):
+        if not THUNDRA_JAVA_AGENT_JAR:
+            return
+
+        thundra_apikey = super(LambdaExecutorLocal, self)._get_thundra_apikey(func_details.envvars)
+        if not thundra_apikey:
+            return
+
+        func_details.envvars[THUNDRA_APIKEY_ENV_VAR_NAME] = thundra_apikey
+
+        return "-javaagent:" + THUNDRA_JAVA_AGENT_JAR
+
     def execute_java_lambda(self, event, context, main_file, func_details=None):
+        func_details.envvars = func_details.envvars or {}
+        given_opts = config.LAMBDA_JAVA_OPTS or ""
+
+        extra_opts = self.inject_thundra_java_agent(func_details, given_opts)
+
+        opts = given_opts
+        if extra_opts:
+            opts += " " + extra_opts
+
         handler = func_details.handler
-        opts = config.LAMBDA_JAVA_OPTS if config.LAMBDA_JAVA_OPTS else ""
+        func_details.envvars[LAMBDA_HANDLER_ENV_VAR_NAME] = handler
+
         event_file = EVENT_FILE_PATTERN.replace("*", short_uid())
         save_file(event_file, json.dumps(json_safe(event)))
         TMP_FILES.append(event_file)
-        class_name = handler.split("::")[0]
+
         classpath = "%s:%s:%s" % (
             main_file,
             Util.get_java_classpath(main_file),
             LAMBDA_EXECUTOR_JAR,
         )
-        cmd = "java %s -cp %s %s %s %s" % (
+        cmd = "java %s -cp %s %s %s" % (
             opts,
             classpath,
             LAMBDA_EXECUTOR_CLASS,
-            class_name,
             event_file,
         )
+
         LOG.info(cmd)
-        result = self._execute_in_custom_runtime(cmd, func_details=func_details)
-        return result
+
+        return self._execute_in_custom_runtime(cmd, func_details=func_details)
 
     def execute_javascript_lambda(self, event, context, main_file, func_details=None):
         handler = func_details.handler
         function = handler.split(".")[-1]
-        event_json_string = "%s" % (json.dumps(event) if event else "{}")
+        event_json_string = "%s" % (json.dumps(json_safe(event)) if event else "{}")
         context_json_string = "%s" % (json.dumps(context.__dict__) if context else "{}")
         cmd = (
             "node -e 'require(\"%s\").%s(%s,%s).then(r => process.stdout.write(JSON.stringify(r)))'"
