@@ -389,6 +389,10 @@ def terminate_all_processes_in_docker():
 
 def serve_resource_graph(data):
     data = json.loads(to_str(data or "{}"))
+
+    if not data.get("awsEnvironment"):
+        raise ValueError("cannot parse aws Environment from empty string")
+
     env = Environment.from_string(data.get("awsEnvironment"))
     graph = dashboard_infra.get_graph(
         name_filter=data.get("nameFilter") or ".*",
