@@ -12,7 +12,7 @@ import time
 import requests
 
 from localstack import config
-from localstack.config import KINESIS_PROVIDER
+from localstack.config import KINESIS_PROVIDER, is_env_true
 from localstack.constants import (
     DEFAULT_SERVICE_PORTS,
     DYNAMODB_JAR_URL,
@@ -28,9 +28,11 @@ from localstack.constants import (
     LOCALSTACK_MAVEN_VERSION,
     MODULE_MAIN_PATH,
     STS_JAR_URL,
+    THUNDRA_APIKEY,
+    THUNDRA_JAVA_AGENT_JAR_NAME,
+    THUNDRA_JAVA_AGENT_VERSION,
 )
 from localstack.utils import bootstrap
-from localstack.utils.common import is_windows
 
 if __name__ == "__main__":
     bootstrap.bootstrap_installation()
@@ -40,17 +42,22 @@ from localstack.utils.common import (
     download,
     get_arch,
     is_alpine,
+    is_windows,
     load_file,
     mkdir,
     new_tmp_file,
     parallelize,
+    retry,
     rm_rf,
     run,
+    safe_run,
     save_file,
     untar,
     unzip,
 )
 
+LOG = logging.getLogger(__name__)
+
 INSTALL_DIR_NPM = "%s/node_modules" % MODULE_MAIN_PATH
 INSTALL_DIR_DDB = "%s/dynamodb" % INSTALL_DIR_INFRA
 INSTALL_DIR_KCL = "%s/amazon-kinesis-client" % INSTALL_DIR_INFRA
@@ -58,6 +65,7 @@ INSTALL_DIR_STEPFUNCTIONS = "%s/stepfunctions" % INSTALL_DIR_INFRA
 INSTALL_DIR_KMS = "%s/kms" % INSTALL_DIR_INFRA
 INSTALL_DIR_ELASTICMQ = "%s/elasticmq" % INSTALL_DIR_INFRA
 INSTALL_PATH_LOCALSTACK_FAT_JAR = "%s/localstack-utils-fat.jar" % INSTALL_DIR_INFRA
+INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR = "%s/%s" % (config.TMP_FOLDER, THUNDRA_JAVA_AGENT_JAR_NAME)
 INSTALL_PATH_DDB_JAR = os.path.join(INSTALL_DIR_DDB, "DynamoDBLocal.jar")
 INSTALL_PATH_KCL_JAR = os.path.join(INSTALL_DIR_KCL, "aws-java-sdk-sts.jar")
 INSTALL_PATH_STEPFUNCTIONS_JAR = os.path.join(INSTALL_DIR_STEPFUNCTIONS, "StepFunctionsLocal.jar")
@@ -69,6 +77,10 @@ URL_LOCALSTACK_FAT_JAR = (
     "https://repo1.maven.org/maven2/"
     + "cloud/localstack/localstack-utils/{v}/localstack-utils-{v}-fat.jar"
 ).format(v=LOCALSTACK_MAVEN_VERSION)
+THUNDRA_JAVA_AGENT_JAR = (
+    "https://repo.thundra.io/service/local/artifact/maven/redirect?"
+    + "r=thundra-releases&g=io.thundra.agent&a=thundra-agent-lambda-bootstrap&v={v}"
+).format(v=THUNDRA_JAVA_AGENT_VERSION)
 MARKER_FILE_LIGHT_VERSION = "%s/.light-version" % INSTALL_DIR_INFRA
 IMAGE_NAME_SFN_LOCAL = "amazon/aws-stepfunctions-local"
 ARTIFACTS_REPO = "https://github.com/localstack/localstack-artifacts"
@@ -81,7 +93,7 @@ SFN_PATCH_CLASS_URL = "%s/raw/master/stepfunctions-local-patch/%s" % (
 )
 
 # kinesis-mock version
-KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.1.3"
+KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.1.7"
 KINESIS_MOCK_RELEASE_URL = (
     "https://api.github.com/repos/etspaceman/kinesis-mock/releases/tags/" + KINESIS_MOCK_VERSION
 )
@@ -95,9 +107,6 @@ JAVAC_TARGET_VERSION = "1.8"
 # SQS backend implementation provider - either "moto" or "elasticmq"
 SQS_BACKEND_IMPL = os.environ.get("SQS_PROVIDER") or "moto"
 
-# set up logger
-LOG = logging.getLogger(__name__)
-
 
 def get_elasticsearch_install_version(version=None):
     if config.SKIP_INFRA_DOWNLOADS:
@@ -148,7 +157,21 @@ def install_elasticsearch(version=None):
             plugin_dir = os.path.join(install_dir, "plugins", plugin)
             if not os.path.exists(plugin_dir):
                 LOG.info("Installing Elasticsearch plugin %s" % (plugin))
-                run("%s install -b %s" % (plugin_binary, plugin))
+
+                def try_install():
+                    safe_run([plugin_binary, "install", "-b", plugin])
+
+                # We're occasionally seeing javax.net.ssl.SSLHandshakeException -> add download retries
+                download_attempts = 3
+                try:
+                    retry(try_install, retries=download_attempts - 1, sleep=2)
+                except Exception:
+                    LOG.warning(
+                        "Unable to download Elasticsearch plugin '%s' after %s attempts"
+                        % (plugin, download_attempts)
+                    )
+                    if not os.environ.get("IGNORE_ES_DOWNLOAD_ERRORS"):
+                        raise
 
     # delete some plugins to free up space
     for plugin in ELASTICSEARCH_DELETE_MODULES:
@@ -209,7 +232,11 @@ def install_kinesis_mock():
     is_probably_m1 = system == "darwin" and ("arm64" in version or "arm32" in version)
 
     LOG.debug("getting kinesis-mock for %s %s", system, machine)
-    if (machine == "x86_64" or machine == "amd64") and not is_probably_m1:
+
+    if is_env_true("KINESIS_MOCK_FORCE_JAVA"):
+        # sometimes the static binaries may have problems, and we want to fal back to Java
+        bin_file = "kinesis-mock.jar"
+    elif (machine == "x86_64" or machine == "amd64") and not is_probably_m1:
         if system == "windows":
             bin_file = "kinesis-mock-mostly-static.exe"
         elif system == "linux":
@@ -266,6 +293,7 @@ def install_local_kms():
 def install_stepfunctions_local():
     if not os.path.exists(INSTALL_PATH_STEPFUNCTIONS_JAR):
         # pull the JAR file from the Docker image, which is more up-to-date than the downloadable JAR file
+        # TODO: works only when running on the host, outside of Docker -> add a fallback if running in Docker?
         log_install_msg("Step Functions")
         mkdir(INSTALL_DIR_STEPFUNCTIONS)
         run("{dc} pull {img}".format(dc=config.DOCKER_CMD, img=IMAGE_NAME_SFN_LOCAL))
@@ -296,10 +324,10 @@ def install_stepfunctions_local():
 
 
 def install_dynamodb_local():
-    is_in_alpine = is_alpine()
     if not os.path.exists(INSTALL_PATH_DDB_JAR):
         log_install_msg("DynamoDB")
         # download and extract archive
+        is_in_alpine = is_alpine()
         tmp_archive = os.path.join(tempfile.gettempdir(), "localstack.ddb.zip")
         dynamodb_url = DYNAMODB_JAR_URL_ALPINE if is_in_alpine else DYNAMODB_JAR_URL
         download_and_extract_with_retry(dynamodb_url, tmp_archive, INSTALL_DIR_DDB)
@@ -351,6 +379,13 @@ def install_lambda_java_libs():
         download(URL_LOCALSTACK_FAT_JAR, INSTALL_PATH_LOCALSTACK_FAT_JAR)
 
 
+def install_thundra_java_agent():
+    # install Thundra Java agent JAR file
+    if not os.path.exists(INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR):
+        log_install_msg("Thundra Java agent", verbatim=True)
+        download(THUNDRA_JAVA_AGENT_JAR, INSTALL_PATH_THUNDRA_JAVA_AGENT_JAR)
+
+
 def install_cloudformation_libs():
     from localstack.services.cloudformation import deployment_utils
 
@@ -375,6 +410,11 @@ def install_component(name):
 def install_components(names):
     parallelize(install_component, names)
     install_lambda_java_libs()
+    # If Thundra API key is specified,
+    # this means that Thundra integration will be used.
+    # So, install Thundra agent.
+    if THUNDRA_APIKEY:
+        install_thundra_java_agent()
 
 
 def install_all_components():
