@@ -2,18 +2,27 @@ import binascii
 import datetime
 import json
 import re
+import xml.etree.ElementTree as ET
 from binascii import crc32
 from struct import pack
+from typing import Optional
 
 import xmltodict
-from flask import Response
+from flask import Response as FlaskResponse
 from requests.models import CaseInsensitiveDict
 from requests.models import Response as RequestsResponse
 
 from localstack.config import DEFAULT_ENCODING
 from localstack.constants import MOTO_ACCOUNT_ID, TEST_AWS_ACCOUNT_ID
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import json_safe, replace_response_content, short_uid, to_bytes, to_str
+from localstack.utils.common import (
+    json_safe,
+    replace_response_content,
+    short_uid,
+    to_bytes,
+    to_str,
+    truncate,
+)
 
 REGEX_FLAGS = re.MULTILINE | re.DOTALL
 
@@ -25,7 +34,9 @@ class ErrorResponse(Exception):
         self.response = response
 
 
-def flask_error_response_json(msg, code=500, error_type="InternalFailure"):
+def flask_error_response_json(
+    msg: str, code: Optional[int] = 500, error_type: Optional[str] = "InternalFailure"
+):
     result = {
         "Type": "User" if code < 500 else "Server",
         "message": msg,
@@ -34,7 +45,7 @@ def flask_error_response_json(msg, code=500, error_type="InternalFailure"):
     headers = {"x-amzn-errortype": error_type}
     # Note: don't use flask's make_response(..) or jsonify(..) here as they
     # can lead to "RuntimeError: working outside of application context".
-    return Response(json.dumps(result), status=code, headers=headers)
+    return FlaskResponse(json.dumps(result), status=code, headers=headers)
 
 
 def requests_error_response_json(message, code=500, error_type="InternalFailure"):
@@ -43,7 +54,11 @@ def requests_error_response_json(message, code=500, error_type="InternalFailure"
 
 
 def requests_error_response_xml(
-    message, code=400, code_string="InvalidParameter", service=None, xmlns=None
+    message: str,
+    code: Optional[int] = 400,
+    code_string: Optional[str] = "InvalidParameter",
+    service: Optional[str] = None,
+    xmlns: Optional[str] = None,
 ):
     response = RequestsResponse()
     xmlns = xmlns or "http://%s.amazonaws.com/doc/2010-03-31/" % service
@@ -59,19 +74,49 @@ def requests_error_response_xml(
     return response
 
 
-def requests_response_xml(action, response, xmlns=None, service=None):
+def to_xml(data: dict, memberize: bool = True) -> ET.Element:
+    """Generate XML element hierarchy out of dict. Wraps list items in <member> tags by default"""
+    if not isinstance(data, dict) or len(data.keys()) != 1:
+        raise Exception("Expected data to be a dict with a single root element")
+
+    def _to_xml(parent_el: ET.Element, data_rest) -> None:
+        if isinstance(data_rest, list):
+            for i in data_rest:
+                member_el = ET.SubElement(parent_el, "member") if memberize else parent_el
+                _to_xml(member_el, i)
+        elif isinstance(data_rest, dict):
+            for key in data_rest:
+                value = data_rest[key]
+                curr_el = ET.SubElement(parent_el, key)
+                _to_xml(curr_el, value)
+        elif isinstance(data_rest, str):
+            parent_el.text = data_rest
+        elif any(
+            [isinstance(data_rest, i) for i in [bool, str, int, float]]
+        ):  # limit types for text serialization
+            parent_el.text = str(data_rest)
+        else:
+            if data_rest is not None:  # None is just ignored and omitted
+                raise Exception(f"Unexpected type for value encountered: {type(data_rest)}")
+
+    root_key = list(data.keys())[0]
+    root = ET.Element(root_key)
+    _to_xml(root, data[root_key])
+    return root
+
+
+def requests_response_xml(action, response, xmlns=None, service=None, memberize=True):
     xmlns = xmlns or "http://%s.amazonaws.com/doc/2010-03-31/" % service
     response = json_safe(response)
     response = {"{action}Result".format(action=action): response}
-    response = xmltodict.unparse(response)
-    if response.startswith("<?xml"):
-        response = re.sub(r"<\?xml [^\?]+\?>", "", response)
+    response = ET.tostring(to_xml(response, memberize=memberize), short_empty_elements=True)
+    response = to_str(response)
     result = (
         """
         <{action}Response xmlns="{xmlns}">
             {response}
         </{action}Response>
-    """
+        """
     ).strip()
     result = result.format(action=action, xmlns=xmlns, response=response)
     result = requests_response(result)
@@ -105,7 +150,6 @@ def requests_error_response_xml_signature_calculation(
     response.status_code = code
 
     if signature and string_to_sign or code_string == "SignatureDoesNotMatch":
-
         bytes_signature = binascii.hexlify(bytes(signature, encoding="utf-8"))
         parsed_response["Error"]["Code"] = code_string
         parsed_response["Error"]["AWSAccessKeyId"] = aws_access_token
@@ -115,7 +159,6 @@ def requests_error_response_xml_signature_calculation(
         set_response_content(response, xmltodict.unparse(parsed_response))
 
     if expires and code_string == "AccessDenied":
-
         server_time = datetime.datetime.utcnow().isoformat()[:-4]
         expires_isoformat = datetime.datetime.fromtimestamp(int(expires)).isoformat()[:-4]
         parsed_response["Error"]["Code"] = code_string
@@ -124,7 +167,6 @@ def requests_error_response_xml_signature_calculation(
         set_response_content(response, xmltodict.unparse(parsed_response))
 
     if not signature and not expires and code_string == "AccessDenied":
-
         set_response_content(response, xmltodict.unparse(parsed_response))
 
     if response._content:
@@ -132,7 +174,11 @@ def requests_error_response_xml_signature_calculation(
 
 
 def flask_error_response_xml(
-    message, code=500, code_string="InternalFailure", service=None, xmlns=None
+    message: str,
+    code: Optional[int] = 500,
+    code_string: Optional[str] = "InternalFailure",
+    service: Optional[str] = None,
+    xmlns: Optional[str] = None,
 ):
     response = requests_error_response_xml(
         message, code=code, code_string=code_string, service=service, xmlns=xmlns
@@ -158,6 +204,23 @@ def requests_error_response(
     )
 
 
+def raise_exception_if_error_response(response):
+    if not is_response_obj(response):
+        return
+    if response.status_code < 400:
+        return
+    content = "..."
+    try:
+        content = truncate(to_str(response.content or ""))
+    except Exception:
+        pass  # ignore if content has non-printable bytes
+    raise Exception("Received error response (code %s): %s" % (response.status_code, content))
+
+
+def is_response_obj(result):
+    return isinstance(result, (RequestsResponse, RequestsResponse))
+
+
 def requests_response(content, status_code=200, headers={}):
     resp = RequestsResponse()
     content = json.dumps(content) if isinstance(content, dict) else content
@@ -182,7 +245,7 @@ def flask_to_requests_response(r):
 
 
 def requests_to_flask_response(r):
-    return Response(r.content, status=r.status_code, headers=dict(r.headers))
+    return FlaskResponse(r.content, status=r.status_code, headers=dict(r.headers))
 
 
 def flask_not_found_error(msg=None):
@@ -215,7 +278,6 @@ def make_error(*args, **kwargs):
 
 
 def create_sqs_system_attributes(headers):
-
     system_attributes = {}
     if "X-Amzn-Trace-Id" in headers:
         system_attributes["AWSTraceHeader"] = {
@@ -226,8 +288,6 @@ def create_sqs_system_attributes(headers):
 
 
 def extract_tags(req_data):
-    keys = []
-    values = []
     for param_name in ["Tag", "member"]:
         keys = extract_url_encoded_param_list(req_data, "Tags.{}.%s.Key".format(param_name))
         values = extract_url_encoded_param_list(req_data, "Tags.{}.%s.Value".format(param_name))
