@@ -3,6 +3,7 @@ import json
 import logging
 import re
 import traceback
+from typing import Optional
 from urllib.parse import urlparse
 
 from moto.cloudformation import parsing
@@ -942,13 +943,14 @@ def resolve_refs_recursively(stack_name, value, resources):
 def resolve_placeholders_in_string(result, stack_name=None, resources=None):
     def _replace(match):
         parts = match.group(1).split(".")
-        if len(parts) == 2:
+        if len(parts) >= 2:
+            resource_name, _, attr_name = match.group(1).partition(".")
             resolved = resolve_ref(
-                stack_name, parts[0].strip(), resources, attribute=parts[1].strip()
+                stack_name, resource_name.strip(), resources, attribute=attr_name.strip()
             )
             if resolved is None:
                 raise DependencyNotYetSatisfied(
-                    resource_ids=parts[0],
+                    resource_ids=resource_name,
                     message="Unable to resolve attribute ref %s" % match.group(1),
                 )
             return resolved
@@ -1514,6 +1516,7 @@ def is_none_or_empty_value(value):
     return not value or value == PLACEHOLDER_AWS_NO_VALUE
 
 
+# TODO: this shouldn't be called for stack parameters
 def determine_resource_physical_id(
     resource_id, resources=None, stack=None, attribute=None, stack_name=None
 ):
@@ -2007,25 +2010,44 @@ class TemplateDeployer(object):
             "Resources"
         ][resource_id]
 
-    def apply_parameter_changes(self, old_stack, new_stack):
+    def resolve_param(
+        self, logical_id: str, param_type: str, default_value: Optional[str] = None
+    ) -> Optional[str]:
+        if param_type == "AWS::SSM::Parameter::Value<String>":
+            ssm_client = aws_stack.connect_to_service("ssm")
+            param = ssm_client.get_parameter(Name=default_value)
+            return param["Parameter"]["Value"]
+        return None
+
+    def apply_parameter_changes(self, old_stack, new_stack) -> None:
         parameters = {
-            p["ParameterKey"]: p["ParameterValue"] for p in old_stack.metadata["Parameters"]
+            p["ParameterKey"]: p
+            for p in old_stack.metadata["Parameters"]  # go through current parameter values
         }
 
-        for key, value in new_stack.template["Parameters"].items():
-            parameters[key] = value.get("Default", parameters.get(key))
+        for logical_id, value in new_stack.template["Parameters"].items():
+            default = value.get("Default")
+            provided_param_value = parameters.get(logical_id)
+            param = {
+                "ParameterKey": logical_id,
+                "ParameterValue": provided_param_value if default is None else default,
+            }
+            if default is not None:
+                resolved_value = self.resolve_param(logical_id, value.get("Type"), default)
+                if resolved_value is not None:
+                    param["ResolvedValue"] = resolved_value
 
-        parameters.update(
-            {p["ParameterKey"]: p["ParameterValue"] for p in new_stack.metadata["Parameters"]}
-        )
+            parameters[logical_id] = param
+
+        parameters.update({p["ParameterKey"]: p for p in new_stack.metadata["Parameters"]})
 
         for change_set in new_stack.change_sets:
             parameters.update({p["ParameterKey"]: p for p in change_set.metadata["Parameters"]})
 
-        old_stack.metadata["Parameters"] = [
-            {"ParameterKey": k, "ParameterValue": v} for k, v in parameters.items() if v
-        ]
+        # TODO: unclear/undocumented behavior in implicitly updating old_stack parameter here
+        old_stack.metadata["Parameters"] = [v for v in parameters.values() if v]
 
+    # TODO: fix circular import with cloudformation_api.py when importing Stack here
     def construct_changes(
         self,
         existing_stack,
