@@ -25,7 +25,7 @@ from contextlib import closing
 from datetime import date, datetime, timezone
 from multiprocessing.dummy import Pool
 from queue import Queue
-from typing import Callable, List, Union
+from typing import Callable, List, Optional, Sized, Union
 from urllib.parse import parse_qs, urlparse
 
 import dns.resolver
@@ -142,7 +142,7 @@ class ShellCommandThread(FuncThread):
                 or not self.process
                 or self.process.returncode == 0
             ):
-                return
+                return self.process.returncode if self.process else None
             LOG.info(
                 "Restarting process (received exit code %s): %s"
                 % (self.process.returncode, self.cmd)
@@ -199,6 +199,7 @@ class ShellCommandThread(FuncThread):
             else:
                 self.process.communicate()
         except Exception as e:
+            self.result_future.set_exception(e)
             if self.process and not self.quiet:
                 LOG.warning('Shell command error "%s": %s' % (e, self.cmd))
         if self.process and not self.quiet and self.process.returncode != 0:
@@ -921,8 +922,10 @@ def format_bytes(count, default="n/a"):
     if not is_number(count) or count < 0:
         return default
     cnt = float(count)
-    for unit in ("B", "KB", "MB", "GB", "TB"):
-        if cnt < 1000:
+    units = ("B", "KB", "MB", "GB", "TB")
+    for unit in units:
+        if cnt < 1000 or unit == units[-1]:
+            # FIXME: will return '1e+03TB' for 1000TB
             return "%s%s" % (format_number(cnt, decimals=3), unit)
         cnt = cnt / 1000.0
     return count
@@ -1726,5 +1729,13 @@ def replace_response_content(response, pattern, replacement):
     response._content = re.sub(pattern, replacement, content)
 
 
+def is_none_or_empty(obj: Union[Optional[str], Optional[list]]) -> bool:
+    return (
+        obj is None
+        or (isinstance(obj, str) and obj.strip() == "")
+        or (isinstance(obj, Sized) and len(obj) == 0)
+    )
+
+
 # Code that requires util functions from above
 CACHE_FILE_PATTERN = CACHE_FILE_PATTERN.replace("_random_dir_", short_uid())
